@@ -33,6 +33,7 @@ from awscli.customizations.s3.results import DryRunResult
 from awscli.customizations.s3.results import ResultRecorder
 from awscli.customizations.s3.results import ResultPrinter
 from awscli.customizations.s3.results import OnlyShowErrorsResultPrinter
+from awscli.customizations.s3.results import NoProgressResultPrinter
 from awscli.customizations.s3.results import ResultProcessor
 from awscli.customizations.s3.results import CommandResultRecorder
 from awscli.customizations.s3.utils import RequestParamsMapper
@@ -45,7 +46,7 @@ from awscli.customizations.s3.utils import DirectoryCreatorSubscriber
 from awscli.customizations.s3.utils import DeleteSourceFileSubscriber
 from awscli.customizations.s3.utils import DeleteSourceObjectSubscriber
 from awscli.customizations.s3.utils import DeleteCopySourceObjectSubscriber
-from awscli.compat import binary_stdin
+from awscli.compat import get_binary_stdin
 
 
 LOGGER = logging.getLogger(__name__)
@@ -110,6 +111,8 @@ class S3TransferHandlerFactory(object):
             result_printer = OnlyShowErrorsResultPrinter(result_recorder)
         elif self._cli_params.get('is_stream'):
             result_printer = OnlyShowErrorsResultPrinter(result_recorder)
+        elif not self._cli_params.get('progress'):
+            result_printer = NoProgressResultPrinter(result_recorder)
         else:
             result_printer = ResultPrinter(result_recorder)
         result_processor_handlers.append(result_printer)
@@ -304,13 +307,28 @@ class BaseTransferRequestSubmitter(object):
                         'Object is of storage class GLACIER. Unable to '
                         'perform %s operations on GLACIER objects. You must '
                         'restore the object to be able to the perform '
-                        'operation.' %
-                        fileinfo.operation_name
+                        'operation. See aws s3 %s help for additional '
+                        'parameter options to ignore or force these '
+                        'transfers.' %
+                        (fileinfo.operation_name, fileinfo.operation_name)
                     )
                     self._result_queue.put(warning)
                 return True
         return False
 
+    def _warn_parent_reference(self, fileinfo):
+        # normpath() will use the OS path separator so we
+        # need to take that into account when checking for a parent prefix.
+        parent_prefix = '..' + os.path.sep
+        escapes_cwd = os.path.normpath(fileinfo.compare_key).startswith(
+            parent_prefix)
+        if escapes_cwd:
+            warning = create_warning(
+                fileinfo.compare_key, "File references a parent directory.")
+            self._result_queue.put(warning)
+            return True
+        return False
+
     def _format_src_dest(self, fileinfo):
         """Returns formatted versions of a fileinfos source and destination."""
         raise NotImplementedError('_format_src_dest')
@@ -396,7 +414,7 @@ class DownloadRequestSubmitter(BaseTransferRequestSubmitter):
         return fileinfo.dest
 
     def _get_warning_handlers(self):
-        return [self._warn_glacier]
+        return [self._warn_glacier, self._warn_parent_reference]
 
     def _format_src_dest(self, fileinfo):
         src = self._format_s3_path(fileinfo.src)
@@ -453,6 +471,7 @@ class UploadStreamRequestSubmitter(UploadRequestSubmitter):
             subscribers.append(ProvideSizeSubscriber(int(expected_size)))
 
     def _get_filein(self, fileinfo):
+        binary_stdin = get_binary_stdin()
         return NonSeekableStream(binary_stdin)
 
     def _format_local_path(self, path):
@@ -479,7 +498,7 @@ class DownloadStreamRequestSubmitter(DownloadRequestSubmitter):
 
 
 class DeleteRequestSubmitter(BaseTransferRequestSubmitter):
-    REQUEST_MAPPER_METHOD = None
+    REQUEST_MAPPER_METHOD = RequestParamsMapper.map_delete_object_params
     RESULT_SUBSCRIBER_CLASS = DeleteResultSubscriber
 
     def can_submit(self, fileinfo):
