@@ -27,14 +27,24 @@ logger = logging.getLogger(__name__)
 # download the content (i.e TemplateURL in cloudformation).
 PARAMFILE_DISABLED = set([
     'apigateway.put-integration.uri',
+    'appstream2.create-stack.redirect-url',
+    'appstream2.update-stack.redirect-url',
     'cloudformation.create-stack.template-url',
     'cloudformation.update-stack.template-url',
+    'cloudformation.create-stack-set.template-url',
+    'cloudformation.update-stack-set.template-url',
+    'cloudformation.create-change-set.template-url',
     'cloudformation.validate-template.template-url',
     'cloudformation.estimate-template-cost.template-url',
+    'cloudformation.get-template-summary.template-url',
 
     'cloudformation.create-stack.stack-policy-url',
     'cloudformation.update-stack.stack-policy-url',
     'cloudformation.set-stack-policy.stack-policy-url',
+    # aws cloudformation package --template-file
+    'custom.package.template-file',
+    # aws cloudformation deploy --template-file
+    'custom.deploy.template-file',
 
     'cloudformation.update-stack.stack-policy-during-update-url',
     # We will want to change the event name to ``s3`` as opposed to
@@ -43,10 +53,28 @@ PARAMFILE_DISABLED = set([
     'custom.mv.website-redirect',
     'custom.sync.website-redirect',
 
+    'guardduty.create-ip-set.location',
+    'guardduty.update-ip-set.location',
+    'guardduty.create-threat-intel-set.location',
+    'guardduty.update-threat-intel-set.location',
+    'comprehend.detect-dominant-language.text',
+    'comprehend.batch-detect-dominant-language.text-list',
+    'comprehend.detect-entities.text',
+    'comprehend.batch-detect-entities.text-list',
+    'comprehend.detect-key-phrases.text',
+    'comprehend.batch-detect-key-phrases.text-list',
+    'comprehend.detect-sentiment.text',
+    'comprehend.batch-detect-sentiment.text-list',
+
     'iam.create-open-id-connect-provider.url',
 
     'machinelearning.predict.predict-endpoint',
 
+    'rds.copy-db-cluster-snapshot.pre-signed-url',
+    'rds.create-db-cluster.pre-signed-url',
+    'rds.copy-db-snapshot.pre-signed-url',
+    'rds.create-db-instance-read-replica.pre-signed-url',
+
     'sqs.add-permission.queue-url',
     'sqs.change-message-visibility.queue-url',
     'sqs.change-message-visibility-batch.queue-url',
@@ -61,6 +89,9 @@ PARAMFILE_DISABLED = set([
     'sqs.send-message-batch.queue-url',
     'sqs.set-queue-attributes.queue-url',
     'sqs.purge-queue.queue-url',
+    'sqs.list-queue-tags.queue-url',
+    'sqs.tag-queue.queue-url',
+    'sqs.untag-queue.queue-url',
 
     's3.copy-object.website-redirect-location',
     's3.create-multipart-upload.website-redirect-location',
@@ -68,6 +99,11 @@ PARAMFILE_DISABLED = set([
 
     # Double check that this has been renamed!
     'sns.subscribe.notification-endpoint',
+
+    'iot.create-job.document-source',
+    'translate.translate-text.text',
+
+    'workdocs.create-notification-subscription.notification-endpoint',
 ])
 
 
